using System;
using System.Drawing;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Eco.Handles;
using Eco.Windows.Forms;
using Eco.WinForm;
using Eco.ObjectRepresentation;
using Eco.Services;
using Eco.Persistence;

using DevGroupGettingStartedWithEco.Model;


namespace DevGroupGettingStartedWithEco
{
    /// <summary>
    /// Summary description for CodeRetrieval.
    /// </summary>
    public partial class CodeRetrieval : System.Windows.Forms.Form
    {

        public CodeRetrieval(DevGroupGettingStartedWithEco.DevGroupGettingStartedWithEcoEcoSpace ecoSpace)
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();

            // Set EcoSpace (actually stores it in rhRoot.EcoSpace)
            // Note that you need to set the EcoSpace property for 
            // each ReferenceHandle and VariableHandle you add to 
            // the form.
            rhRoot.EcoSpace = ecoSpace;
        }

        public DevGroupGettingStartedWithEco.DevGroupGettingStartedWithEcoEcoSpace EcoSpace
        {
            get { return (DevGroupGettingStartedWithEco.DevGroupGettingStartedWithEcoEcoSpace)rhRoot.EcoSpace; }
        }

        private void btnGetInvoices_Click(object sender, EventArgs e)
        {
            IOclService ocl = EcoSpace.GetEcoService<IOclService>();
            if (ocl != null)
            {
                IElement result = ocl.Evaluate("Invoice.allInstances");

                // This element should be an IList of typed Invoice's
                IList<Invoice> invoices = result.GetAsIList<Invoice>();
                //IElementCollection invs = result.GetAsCollection();

                if (invoices != null)
                {
                    // then we have a list of Invoices we can pass to something, or set as a datasource for
                    // say a grid or something else.
                    dgvObjects.DataSource = invoices;
                    statusDetails.Text = "OCL Query Evaluated";
                }
            }
        }

        private void btnGet_Click(object sender, EventArgs e)
        {
            // Show an example of doing a Persistence Storage search based on a condition
            // and then using that in a further OCL query.
            IOclPsService ops = EcoSpace.GetEcoService<IOclPsService>();
            if (ops != null)
            {
                AbstractCondition cond = ops.CreateRetrieveCondition(null, null, "Invoice.allInstances->select(Reference = '00000001')");

                IPersistenceService ps = EcoSpace.GetEcoService<IPersistenceService>();
                IObjectList result = ps.GetAllWithCondition(cond);

                // You can now use this result for further filtering
                IOclService os = EcoSpace.GetEcoService<IOclService>();

                // Here we are just going to pass it to an OCL Expression to show we could 
                // perform further operations on the results returned to us previously.
                IElement el = os.Evaluate(result, "self", null);
                IList<Invoice> invoices = el.GetAsIList<Invoice>();
                if (invoices != null)
                {
                    dgvObjects.DataSource = invoices;
                    statusDetails.Text = "OCL PS Condition Queried, Evaluated and Returned";
                }
            }
        }
    }
}
